/***********************************************************************
  file ecalcf.c
  ++++++++++++++++++++

                     Evaluation of an S Function
                     of One  Argument for MINFUN
  ______________________________________________________________________

  This is c code to evaluate an S function f at a vector  x and return
  the value in ans as type float.

  Note that the (void) function name is ecalcf_ so that it can be
  invoked from Fortran.

  f takes one vector argument which must be type float.

  It is suggested that a strategy such as that described on
  page 210 of New S be employed to assure that a single number
  is returned by the user function.

  If this is not done, the user must assure that one
  double value is returned by his S function

                      Arguments

  f --> S function of one argument of type float.  f is
        passed here through a trivial C driver that invokes a
        Fortran routine in which all statements FX = F(X) are changed
        to CALL ECALCF( F, X, FX ).

  x --> value at which f is to be evaluated.
                   x is float

  dimx --> length of x
                   dimx is int

  ans <-- value of f(x).
                   ans is double

***********************************************************************/
#include "S.h"
#include <stdio.h>
void F77_SUB(ecalcf)( f, x, dimx, ans )
char **f; float *x; int *dimx; double *ans;
{  char *func;
   char *args[1]; char *result[1];
   char *mode[1]; long length[1];
   mode[0] = "single"; length[0] = *dimx;
   args[0] = (char *) x;
   func = f[0];
   call_S( func, 1L, args, mode, length, 0, 1L, result);
   *ans = *( (double *) result[0] );
 }
