      SUBROUTINE hess(fn,x,n,scl,stpsz,fnbr,h)
C***********************************************************************
C          -Calculate Single Precision Hessian Approximation-
C                     -Using Only Function Values-
C
C     DESCRIPTION:
C          Calculates a forward difference approximation to the
C          hessian using only the values of a user-supplied function.
C          Algorithm taken from Appendix A of "Numerical Methods for
C          Unconstrained Optimization and Nonlinear Equations" by
C          J.E. Dennis, Jr. and Robert B. Schnabel,  1983,
C          p.321-322.
C          (programmed by Liz Rozell - February, 1985)
C
C     ARGUMENTS:
C
C          FN --> name of the user-supplied function. be sure to declare
C          external in the calling program!
C
C          X --> real array dimensioned X(N)
C          which contains the current parameters of the function
C          at which the hessian is to be calculated.
C
C          N --> Number of parameters. (integer)
C
C          SCL --> real array dimensioned SCL(N)
C          which contains the scaling factors for each parameter.
C          here 1/SCL(i) is the typical size of abs(X(i)).
C
C          H <-- real array dimensioned H(N,N)
C          which contains the calculated hessian values.
C
C          STPSZ --> real array dimensioned STPSZ(N)
C          which contains the stepsize for each parameter.
C          (passed only for storage considerations.)
C
C          FNBR --> real array dimensioned FNBR(N)
C          which contains the function value of the i'th parameter
C          plus the i'th stepsize.
C          (passed only for storage considerations.)
C
C***********************************************************************
C
C     Declarations
C
      IMPLICIT REAL (a-h,o,p,r-z)
C
      REAL x(n),scl(n),stpsz(n),fnbr(n),h(n,n)
      REAL machep
      DOUBLE PRECISION fnval
C
      EXTERNAL fn
C
C     End declarations
C
      machep = r1mach(4)
C      FC = FN(X,N)
      CALL ecalcf(fn,x,n,fnval)
      fc = fnval
      c = machep** (1.0/3.0)
C
C     Calculate stepsize and updated function value
C
      DO 10,i = 1,n
          stpsz(i) = max(abs(x(i)),1.0/scl(i))
          stpsz(i) = stpsz(i)*c*sign(1.0,x(i))
          tempi = x(i)
          x(i) = x(i) + stpsz(i)
          stpsz(i) = x(i) - tempi
C      FNBR(I) = FN(X,N)
          CALL ecalcf(fn,x,n,fnval)
          fnbr(i) = fnval
          x(i) = tempi
   10 CONTINUE
      DO 30,i = 1,n
C
C     Calculate Hessian
C
C
C     Calculate Diagonal Elements
C
          tempi = x(i)
          x(i) = x(i) + 2.0*stpsz(i)
          CALL ecalcf(fn,x,n,fnval)
          fii = fnval
          h(i,i) = ((fc-fnbr(i))+ (fii-fnbr(i)))/ (stpsz(i)*stpsz(i))
          x(i) = tempi + stpsz(i)
          DO 20,j = i + 1,n
C
C     Calculate Off-Diagonal Elements
C
              tempj = x(j)
              x(j) = x(j) + stpsz(j)
              CALL ecalcf(fn,x,n,fnval)
              fij = fnval
              h(i,j) = ((fc-fnbr(i))+ (fij-fnbr(j)))/
     +                 (stpsz(i)*stpsz(j))
              h(j,i) = h(i,j)
              x(j) = tempj
   20     CONTINUE
          x(i) = tempi
   30 CONTINUE
      RETURN
C
      END
