hessian <- function( f, param, scale=rep(1,length(param) ) ) {
#**********************************************************************
#
#            Hessian of a Function of Several Parameters
#
# Function - Packages algorithm FDHESSG from Appendix A (Page 320)
#            of Dennis, JE and Schnabel, RB: Numberical Methods for
#            Unconstrained Optimization and Nonlinear Equations.
#            Prentice-Hall, Inc., Englewood Cliffs, NJ (1983)
#
# Arguments
#
# f : function whose hessian is desired.  f takes as its only argument a
#     vector of parameter values and returns a single numeric
#     value
#
# param : vector of parameter values at which hessian is to be taken
#
# scale (optional): vector of scale factors such that to a rough
#        approximation scale(i) is a typical value for param(i).  
#        For example, if param(1) has values in [1.0E-6,1.0E-5]
#        and param(2) has values in [10,100], then an appropriate 
#        value for scale would be c(5.0e-6,50).  Not important unless
#        the typical values differ a great deal.
#
#**********************************************************************
#
# Check input argument existence and type
#
# For distribution, user must take care of dyn.load or library.dynam
#
#library.dynam('hessian','rhess.o')
if ( missing(f) || !is.function(f) )
   stop(' hess - first argument must be function')
if ( missing(param) || !is.numeric(param) ||
     length(param) < 1 )
  stop(' hess - second argument must be parameters at which hessian estimated')
npar <- length(param)
if ( !missing(scale) && ( !is.numeric(scale) ||
                            length(scale) != npar ) )
   stop(' hess - scale must be numeric same size as param')
#
# define fn.check as per page 210 of New S book
#
fn.check <- function(x){
   x <- Usr.fn(x)
   if ( !is.numeric(x) )
      stop( 'function in hess must return numeric value' )
      as.double(x)     }
   assign("Usr.fn", f, frame=1)
# invoke c driver for hess
#
z <- .C("chess",                    # routine called
        list(fn.check),             # function to be evaluated
        as.single( param ),  
        as.integer( npar ),
        as.single( scale ),
        single( npar ),            # space for bounds array
        single( npar ),            # space for bounds array
        ans=single(npar*npar)      # hessian
        )
return( matrix(z$ans,nrow=npar,ncol=npar) )
}
